/*
 * Copyright (C) 2012, Eric Wong <normalperson@yhbt.net>
 * License: GPLv3 or later (see COPYING for details)
 */
#include "cmogstored.h"

/* path must be a free()-able pointer */
struct mog_fd *
mog_file_open_read(struct mog_svc *svc, char *path)
{
	struct mog_fd *mfd;
	struct mog_file *mfile;
	int fd = mog_open_read(svc, path);

	if (fd < 0) return NULL;

	mfd = mog_fd_get(fd);
	mfd->fd = fd;
	mfd->fd_type = MOG_FD_TYPE_FILE;
	mfd->queue_state = MOG_QUEUE_STATE_NEW;

	mfile = &mfd->as.file;
	memset(mfile, 0, sizeof(struct mog_file));
	mfile->fsize = -1;
	mfile->svc = svc;
	mfile->path = path;

	return mfd;
}

int mog_file_fstat(struct mog_fd *mfd, struct stat *sb)
{
	assert(mfd->fd_type == MOG_FD_TYPE_FILE && "mog_fd is not a file");

	return fstat(mfd->fd, sb);
}

void mog_file_close(struct mog_fd *mfd)
{
	struct mog_file *mfile = &mfd->as.file;

	assert(mfd->fd_type == MOG_FD_TYPE_FILE && "mog_fd is not a file");

	mog_close(mfd->fd);
	assert(mfile->path && "mog_file->path unset!");
	free(mfile->path);
	free(mfile->tmppath); /* may be NULL */
	free(mfile->md5);
}

size_t mog_file_pathlen(struct mog_fd *mfd)
{
	struct mog_file *mfile = &mfd->as.file;

	if (mfile->pathlen == 0)
		mfile->pathlen = strlen(mfile->path);

	return mfile->pathlen;
}
