/*
 * Copyright (C) 2012, Eric Wong <normalperson@yhbt.net>
 * License: GPLv3 or later (see COPYING for details)
 */
#include "cmogstored.h"

%%{
	machine iostat_parser;
	eor = '\n'> { iostat->line_end = fpc - buf; };
	ignored_line := ( (any-'\n')* eor ) @ { fgoto main; };
	device = "Device:" (any-'\n')* eor;
	stats = space*
		(
			((any - space)+) > {
				iostat->dev_tip = fpc - buf;
				iostat->dev_len = 0;
				iostat->util_tip = 0;
				iostat->util_len = 0;
			}
			((space - '\n')+) > {
				iostat->dev_len = fpc - buf - iostat->dev_tip;
			}
		)

		# Skip the middle section for now, some folks may use
		# await/svctm here.  Not sure how standardized those
		# fields are on non-Linux platforms...
		(any - '\n')*
		(space - '\n')+

		((any - space)+) > { iostat->util_tip = fpc - buf; }
		eor > {
			iostat->util_len = fpc - buf - iostat->util_tip;
			mog_iostat_line_done(iostat, buf, len);
		};
	blank = (space*)eor;
	dev_stats = (device stats+ blank)+ @ { mog_iostat_commit(); };
	main := dev_stats $! { fhold; fgoto ignored_line; };
}%%

%% write data;

void mog_iostat_init(struct mog_iostat *iostat)
{
	int cs;
	struct mog_queue *queue = iostat->queue;

	memset(iostat, 0, sizeof(struct mog_iostat));
	%% write init;
	iostat->cs = cs;
	iostat->queue = queue;
}

enum mog_iostat_parser_state
mog_iostat_parse(struct mog_iostat *iostat, char *buf, size_t len)
{
	char *p, *pe, *eof = NULL;
	int cs = iostat->cs;
	size_t off = iostat->offset;

	assert(off <= len && "iostat offset past end of buffer");

	if (cs == iostat_parser_first_final)
		return MOG_IOSTAT_PARSER_DONE;

	p = buf + off;
	pe = buf + len;

	assert((void *)(pe - p) == (void *)(len - off) &&
	       "pointers aren't same distance");

	%% write exec;

	iostat->cs = cs;
	iostat->offset = p - buf;

	if (cs == iostat_parser_error)
		return MOG_IOSTAT_PARSER_ERROR;

	assert(p <= pe && "buffer overflow after iostat parse");
	assert(iostat->offset <= len && "offset longer than len");

	if (cs == iostat_parser_first_final)
		return MOG_IOSTAT_PARSER_DONE;

	return MOG_IOSTAT_PARSER_WANTREAD;
}
