/*
 * Copyright (C) 2012, Eric Wong <normalperson@yhbt.net>
 * License: GPLv3 or later (see COPYING for details)
 */
#include "cmogstored.h"

/* we can lower this if we can test with lower values, but NPTL is 16K */
#define MOG_THR_STACK_SIZE ((16 * 1024))

#if defined(PTHREAD_STACK_MIN) && (PTHREAD_STACK_MIN > MOG_THR_STACK_SIZE)
#  undef MOG_THR_STACK_SIZE
#  define MOG_THR_STACK_SIZE PTHREAD_STACK_MIN
#endif

static const size_t stacksize = (size_t)MOG_THR_STACK_SIZE;

void
mog_thrpool_start(struct mog_thrpool *tp, size_t n,
                  void *(*start_fn)(void *), void *arg)
{
	size_t i;
	pthread_t *thr;

	tp->threads = thr = xmalloc(sizeof(pthread_t) * n);
	tp->n_threads = n;

	for (i = 0; i < n; i++, thr++) {
		pthread_attr_t attr;

		CHECK(int, 0, pthread_attr_init(&attr));
		CHECK(int, 0, pthread_attr_setstacksize(&attr, stacksize));
		CHECK(int, 0, pthread_create(thr, &attr, start_fn, arg));
		CHECK(int, 0, pthread_attr_destroy(&attr));
	}
}

void mog_thrpool_quit(struct mog_thrpool *tp)
{
	pthread_t *thr = tp->threads;

	for (; tp->n_threads--; thr++) {
		CHECK(int, 0, pthread_cancel(*thr));
		CHECK(int, 0, pthread_join(*thr, NULL));
	}
	mog_free_and_null(&tp->threads);
}
